/******************************************************************************
***
***				COPYRIGHT
***
***	Die Daten in dieser Datei sind Eigentum der 
***
***	YARD Software GmbH in 5000 Koeln 90, Deutschland.
***
***	Diese Datei gilt als Werk im Sinne des Urheberrechtsgesetzes und ist
***	damit urheberrechtlich geschuetzt. Jede Verwertung, insbesondere die
***	Vervielfaeltigung, Aenderung und Weitergabe, auch auszugsweise,
***	ist nur mit ausdruecklicher schriftlicher Genehmigung der YARD
***	Software GmbH zulaessig.
***
***	Ebenso ist die Speicherung dieser Datei auf Datentraegern wie z.B.
***	Festplatten, Disketten, Magnetbaendern, CDs und aehnlichen, auch 
***	auszugsweise, nur mit ausdruecklicher schriftlicher Genehmigung der 
***	YARD Software GmbH zulaessig.
***
***	Zuwiderhandlungen unterliegen den Strafbestimmungen des Urheberrechts-
***	gesetzes.
***
******************************************************************************/

#include <stdio.h>

#include "clibasic.h"
#include "sql.h"
#include "sqlext.h"

#include "sqlda.h"

/******************************************************************************
***
***	the following defines are used for easier error handling through the
***	routine fatal().
***
******************************************************************************/

#define ENV_HANDLE	1
#define CON_HANDLE	2
#define STM_HANDLE	3

/******************************************************************************
***
***	Prototypes
***
******************************************************************************/

void do_select(HDBC,char*);
HDBC do_connect(HDBC,char*);
void do_opendb(HDBC,char*,char*);
sqlda_t *get_descriptor(HSTMT,short);
void bind_columns(HSTMT,sqlda_t*);
void print_da(sqlda_t*);
char *get_c_typename(short);
void fatal(char *, int , char *, void *, int );
void do_disconnect(HDBC );
void print_data(sqlda_t *);

/******************************************************************************
***
***	CLI demo program.
***
******************************************************************************/

RETCODE sqlret;

main(int argc, char **argv)
{
	char *dbname = "RENTAL";
	char *prog = argv[0];
	int choice = -1;
	register int i;
	HDBC hdbc;
	HENV henv;

	/*
	**	Allocate the required handles and connect to the DBS
	**	The setting of YARDDBS will be used.
	*/

	if ((sqlret = SQLAllocEnv(&henv)))
		fatal(__FILE__, __LINE__, "SQLAllocEnv()", NULL, ENV_HANDLE);

	hdbc = do_connect(henv,"yard3@yard");

	/*
	**	open the database dbname
	*/

	do_opendb(hdbc,dbname,prog);

	do_select(hdbc, "select * from customer");

	/*
	**	Disconnect from the database and
	**	free all allocated statements
	*/

	do_disconnect(hdbc);
	
	if ((sqlret = SQLFreeConnect(hdbc)))
		fatal(__FILE__, __LINE__, "SQLFreeConnect()", hdbc, CON_HANDLE);

	if ((sqlret = SQLFreeEnv(henv)))
		fatal(__FILE__, __LINE__, "SQLFreeEnv()", henv, ENV_HANDLE);

 	exit(0);
}

/******************************************************************************
***
***
***
******************************************************************************/

void do_select(HDBC hdbc, char *stmt)
{
	short i,rows;
	short ncols;
	sqlda_t *da;
	sqlha_t *ha;
	HSTMT hstmt;

	if ((sqlret = SQLAllocStmt(hdbc, &hstmt)))
		fatal(__FILE__, __LINE__, "SQLAllocStmt()", hdbc, CON_HANDLE);

	/*
	**	Process the statement
	*/

	if ((sqlret = SQLExecDirect(hstmt, stmt, SQL_NTS)))
		fatal(__FILE__, __LINE__, "SQLExecDirect()", hstmt, STM_HANDLE);

	/*
	**	Get the number of result cols
	*/

	if ((sqlret = SQLNumResultCols(hstmt, &ncols)))
		fatal(__FILE__, __LINE__, "SQLNumResultCols()", hstmt, 
			STM_HANDLE);

	/*
	**	Build up a descriptor to fetch the result data,
	**	print the descriptor contents for debugging purpose,
	**	and bind all columns with SQLBindCol()
	*/

	da = get_descriptor(hstmt, ncols);
	print_da(da);
	bind_columns(hstmt, da);
	
	/*
	**	Fetch the data using SQLFetch() until there are
	**	no more rows.
	**	The result data of unbound columns (binary data, see above) 
	**	are stored with SQLGetData() which is called within the
	**	function get_blob_data().
	**	See this function for further comments.
	*/

	rows = 0;
	for (;;)
	{
		switch (sqlret = SQLFetch(hstmt))
		{
			case SQL_INVALID_HANDLE:
			case SQL_ERROR:
				fatal(__FILE__, __LINE__, "SQLFetch()", hstmt, 
					STM_HANDLE);
				break;
			case SQL_NO_DATA_FOUND:
				break;
			default:
				print_data(da);
		}
		if (sqlret == SQL_NO_DATA_FOUND)
			break;
		rows++;
	}

	if ((sqlret = SQLFreeStmt(hstmt, SQL_DROP)))
		fatal(__FILE__, __LINE__, "SQLFreeStmt()", hstmt, STM_HANDLE);

	printf("%d rows found\n",rows);
 	return;
}

/******************************************************************************
***
***
***
******************************************************************************/

sqlda_t *get_descriptor(HSTMT hstmt, short ncols)
{
	int i;
	sqlda_t *da = (sqlda_t *)calloc(1, sizeof(sqlda_t));
	sqlha_t *ha;

	if (!da)
		fatal(__FILE__, __LINE__, "get_descriptor(): malloc error", 
			NULL, 0);

	da->hostcnt = (long)ncols;
	da->sqlha = (sqlha_t *)calloc((int)ncols, sizeof(sqlha_t));
	ha = da->sqlha;

	for (i = 0; i < (int)ncols; i++, ha++)
	{
		if ((sqlret = SQLDescribeCol(hstmt, (short)(i+1), ha->colname,
			sizeof(ha->colname), NULL, &ha->type,
			&ha->precision, &ha->scale, &ha->nulls)))
			fatal(__FILE__, __LINE__, "SQLDescribeCol()", hstmt, 
				STM_HANDLE);
		
		switch (ha->type)
		{
			case SQL_VARCHAR:
			case SQL_CHAR:
				ha->type = SQL_C_CHAR;
				break;
			/*
			**	Convert DECIMAL and DATE to CHAR
			*/
			case SQL_DECIMAL:
			case SQL_NUMERIC:
			case SQL_DATE:
				ha->type = SQL_C_CHAR;
				ha->precision = 50;
				break;
			case SQL_FLOAT:
				ha->type = SQL_C_DOUBLE;
				break;
			case SQL_INTEGER:
				ha->type = SQL_C_LONG;
				ha->precision = 10;
				break;
			case SQL_LONGVARCHAR:
			case SQL_LONGVARBINARY:
			case SQL_VARBINARY:
			case SQL_BINARY:
				ha->type = SQL_C_CHAR;
				ha->precision = 32000;
				ha->length = 0;
				break;
			default:
				fatal(__FILE__, __LINE__,
					"get_descriptor(): invalid data type",
					NULL,0);
		}
		ha->data = (char *)calloc(1, ha->precision+1);
		if (!ha->data)
			fatal(__FILE__, __LINE__, "get_descriptor(): malloc error",
				NULL, 0);
		ha->indicator = (long *)calloc(1, sizeof(long));
		if (!ha->data)
			fatal(__FILE__, __LINE__, "get_descriptor(): malloc error", 
				NULL, 0);

		/*
		**	Benutzernamen und Tabellenamen speichern
		*/
		if ((sqlret = SQLColAttributes(hstmt, (short)(i+1),
			SQL_COLUMN_OWNER_NAME, ha->usrname,
			sizeof(ha->usrname), NULL, NULL)))
			fatal(__FILE__, __LINE__, "SQLColAttributes()", hstmt, 
				STM_HANDLE);
		if ((sqlret = SQLColAttributes(hstmt, (short)(i+1),
			SQL_COLUMN_TABLE_NAME, ha->tabname,
			sizeof(ha->tabname), NULL, NULL)))
			fatal(__FILE__, __LINE__, "SQLColAttributes()", hstmt, 
				STM_HANDLE);
	}

	return(da);
}
	
/******************************************************************************
***
***
***
******************************************************************************/

void bind_columns(HSTMT hstmt, sqlda_t *da)
{
	int i;
	sqlha_t *ha = da->sqlha;

	for (i = 0; i < da->hostcnt; i++, ha++)
	{
		if ((sqlret = SQLBindCol(hstmt, (i+1), ha->type, 
			ha->data, ha->precision+1, ha->indicator)))
			fatal(__FILE__, __LINE__, "SQLBindCol()", hstmt, 
				STM_HANDLE);
	}
}

/******************************************************************************
***
***
***
******************************************************************************/

void print_da(sqlda_t *da)
{
	int i;
	sqlha_t *ha = da->sqlha;

	fprintf(stdout, "Usr = <%-14.14s> Table = <%-18.18s>\n",
		ha->usrname, ha->tabname);
	 
	for (i = 0; i < da->hostcnt; i++, ha++)
	{
		fprintf(stdout, "Column = <%-18.18s> Type = %2d %s\n",
			ha->colname, ha->type, get_c_typename(ha->type));
	}
}

/******************************************************************************
***
***
***
******************************************************************************/

void print_data(sqlda_t *da)
{
	int i;
	sqlha_t *ha = da->sqlha;

	for (i = 0; i < da->hostcnt; i++, ha++)
	{
		switch (ha->type)
		{
			case SQL_C_CHAR:
				printf("%s ",ha->data);
				break;
			case SQL_C_LONG:
				printf("%d ",(int)*(int*)ha->data);
				break;
		}
	}
	printf("\n");
}

/******************************************************************************
***
***
***
******************************************************************************/

char *get_c_typename(short ctype)
{
	switch (ctype)
	{
		case SQL_C_CHAR: 
			return("SQL_C_CHAR");
		case SQL_C_LONG:
			return("SQL_C_LONG");
		case SQL_C_SHORT:
			return("SQL_C_SHORT");
			return("SQL_C_SHORT");
		case SQL_C_FLOAT:
			return("SQL_C_FLOAT");
		case SQL_C_DOUBLE:
			return("SQL_C_DOUBLE");
		/*
		**	Invalid value for C data type
		*/

		default:
			return("UNKNOWN");
	}
}

/******************************************************************************
***
***
***
******************************************************************************/

HDBC do_connect(HENV henv, char *dbs)
{
	HDBC hdbc;

	if ((sqlret = SQLAllocConnect(henv, &hdbc)))
		fatal(__FILE__, __LINE__, "SQLAllocConnect()", henv, 
			ENV_HANDLE);

	if ((sqlret = SQLConnect(hdbc, dbs, SQL_NTS,
		"DBA", SQL_NTS, "", SQL_NTS)))
		fatal(__FILE__, __LINE__, "SQLConnect()", hdbc, CON_HANDLE);
	return(hdbc);
}

void do_opendb(HDBC hdbc, char *dbname, char *prog)
{
	char stmt[200];
	HSTMT hstmt;

	sprintf(stmt, "database '%s'", dbname);

	if ((sqlret = SQLAllocStmt(hdbc, &hstmt)))
		fatal(__FILE__, __LINE__, "SQLAllocStmt()", hdbc, CON_HANDLE);

	if ((sqlret = SQLExecDirect(hstmt, stmt, SQL_NTS)))
		fatal(__FILE__, __LINE__, "SQLExecDirect()", hstmt, STM_HANDLE);

	if (prog)
	{
		char logfile[512];
		sprintf(logfile, "%s.log", prog);
		sprintf(stmt, "set logfile to '%s'", logfile);
		if ((sqlret = SQLExecDirect(hstmt, stmt, SQL_NTS)))
			fatal(__FILE__, __LINE__, "SQLExecDirect()", hstmt, 
				STM_HANDLE);
	}

	if ((sqlret = SQLFreeStmt(hstmt, SQL_DROP)))
		fatal(__FILE__, __LINE__, "SQLFreeStmt()", hstmt, STM_HANDLE);
}

/******************************************************************************
***
***
***
******************************************************************************/

void do_disconnect(HDBC hdbc)
{

	if ((sqlret = SQLDisconnect(hdbc)))
		fatal(__FILE__, __LINE__, "SQLDisconnect()", hdbc, CON_HANDLE);

}

/******************************************************************************
***
***
***
******************************************************************************/

void fatal(char *module, int line, char *func, void *handle, int hndflg)
{
	HENV env = NULL;
	HDBC con = NULL;
	HSTMT stm = NULL;
	SCHAR sqlstate1[6];
	SCHAR sqlstate2[6];
	UDWORD sqlcode, isamcode;
	SCHAR errmsg1[512], errmsg2[512];
	SWORD errsize1, errsize2;
	RETCODE ret;

	if (sqlret == SQL_INVALID_HANDLE)
	{
		fprintf(stderr, "%s: (%d) function '%s': INVALID HANDLE\n",
			module, line, func);
		exit(1);
	}

	/*
	**	No handle, internal error, eg. malloc()
	*/

	if (!handle)
	{
		fprintf(stderr, "%s: (%d) function '%s'\n", module, line, func);
		exit(1);
	}

	fprintf(stderr, "%s: (%d) function %s:\n", module, line, func);

	if (!handle)
		exit(1);

	switch (hndflg)
	{
		case ENV_HANDLE:
			env = (HENV)handle;
			break;
		case CON_HANDLE:
			con = (HDBC)handle;
			break;
		case STM_HANDLE:
			stm = (HSTMT)handle;
			break;
	}

	ret = SQLError(env, con, stm, sqlstate1, &sqlcode, errmsg1, 
		sizeof(errmsg1), &errsize1);
	if (ret == SQL_NO_DATA_FOUND)
		exit(1);
	ret = SQLError(env, con, stm, sqlstate2, &isamcode, errmsg2, 
		sizeof(errmsg2), &errsize2);

	fprintf(stderr, "SQLSTATE= %s\n", sqlstate1);
	fprintf(stderr, "SQLCODE = %d\n", sqlcode);
	fprintf(stderr, "SQLMSG  = %s\n", errmsg1);
	if (ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO)
	{
		fprintf(stderr, "SQLSTATE= %s\n", sqlstate2);
		fprintf(stderr, "ISAMCODE= %d\n", isamcode);
		fprintf(stderr, "SQLMSG  = %s\n", errmsg2);
	}

	exit(1);
}
