/******************************************************************************
***
***				COPYRIGHT
***
***	Die Daten in dieser Datei sind Eigentum der 
***
***	YARD Software GmbH, 51107 Koeln, Deutschland.
***
***	Diese Datei gilt als Werk im Sinne des Urheberrechtsgesetzes und ist
***	damit urheberrechtlich geschuetzt. Jede Verwertung, insbesondere die
***	Vervielfaeltigung, Aenderung und Weitergabe, auch auszugsweise,
***	ist nur mit ausdruecklicher schriftlicher Genehmigung der YARD
***	Software GmbH zulaessig.
***
***	Ebenso ist die Speicherung dieser Datei auf Datentraegern wie z.B.
***	Festplatten, Disketten, Magnetbaendern, CDs und aehnlichen, auch 
***	auszugsweise, nur mit ausdruecklicher schriftlicher Genehmigung der 
***	YARD Software GmbH zulaessig.
***
***	Zuwiderhandlungen unterliegen den Strafbestimmungen des Urheberrechts-
***	gesetzes.
***
******************************************************************************/

static char *demo5_ec_sccsid = "@(#) demo5.ec 1.1 98/11/05 16:44:29";

/*
**	This program is an example for handling BLOB data
**	from within an ESQLC program.
**	It shows an UPDATE and SELECT of a BLOB column.
**	The BLOB data is located in a named file (UPDATE).
**	The SELECT will locate the BLOB data into main memory.
**	
**	PREREQUISITES:
**	
**	The database RENTAL must be installed using 'yarddemo'
**	The environment variable YARDDBS must be set.
*/

static int update_picture(void);
static int select_picture(void);

#include <stdio.h>

/*
**	Include the ESQLC header file
**	for handling BLOB data
*/
#include "sqlblob.h"

main(int argc, char **argv)
{
/*
**	Hostvariables to hold the
**	select results.
*/
exec sql begin declare section;
	char lname[21];
	char fname[21];
	char state[21];
	char cartype[21];
exec sql end declare section;

	/*
	**	The program will stop in case of an error
	*/
	exec sql whenever sqlerror stop;

	/*
	**	Connect to the database server with user DBA
	**	The DBS name is taken from the environment variable
	**	YARDDBS during connect.
	*/
	exec sql connect user dba;

	/*
	**	Open the database RENTAL
	*/
	exec sql database rental;

	/*
	**	Update one picture column
	**	in table car
	*/
	update_picture();

	/*
	**	Select the picture
	**	into a named file.
	*/
	select_picture();

	exit(0);
}

static int update_picture(void)
{
exec sql begin declare section;

        blob_t new_picture;

exec sql end declare section;

        register int i;

	/*
	**	The file, which contains
	**	the picture.
	*/
        char *blobfname = "e_class_med.jpg";
	
        /*
        **      Initialize the BLOB structure,
	**	so that the data can be located
	**	from a named file.
        */
        new_picture.b_type = BLOB_FILENAME;
        new_picture.b_loc.b_file.b_fname = blobfname;

	/*
	**	Update the picture column
	**	for Mercedes E 320.
	**	The (unique) regristration is used
	**	to locate the car.
	*/
	exec sql update car set picture = :new_picture
		where registration = 'S-LY-1655';

        exec sql commit;

        return(0);
}

static int select_picture(void)
{
exec sql begin declare section;

        blob_t new_picture;

exec sql end declare section;

        register int i;
	
        /*
        **      Initialize the BLOB structure,
	**	so that the data will be located
	**	in memory.
 	**	The memory will be allocated by
	**	then runtime system (libsql.a).
        */
        new_picture.b_type = BLOB_SYSMEM;

	/*
	**	Select the picture column
	**	for Mercedes E 320.
	**	A subselect on table cartype is
	**	used to locate the column.
	**	The (unique) regristration is used
	**	to locate the car.
	**	We can do a single Select without
	**	using a cursor, because we know that
	**	only one row is returned.
	*/
	exec sql select picture 
		into :new_picture
		from car
		where registration = 'S-LY-1655';

	/*
	**	The blob data is now located in
	**	the element b_buf of the b_memory
	**	structure within the blob structure.
	**	The element b_size contains the size
	**	of the data.
	**	It is left to the user to process the
	**	blob data (eg: write it to a file or
	**	something like that).
	**	In this program only the size and the
	**	adress of buffer (b_buf) is displayed.
	*/
	printf("size: %d buf: 0x%x\n",
		new_picture.b_loc.b_memory.b_size,
		new_picture.b_loc.b_memory.b_buf);

        return(0);
}
