/******************************************************************************
***
***				COPYRIGHT
***
***	Die Daten in dieser Datei sind Eigentum der 
***
***	YARD Software GmbH in 5000 Koeln 90, Deutschland.
***
***	Diese Datei gilt als Werk im Sinne des Urheberrechtsgesetzes und ist
***	damit urheberrechtlich geschuetzt. Jede Verwertung, insbesondere die
***	Vervielfaeltigung, Aenderung und Weitergabe, auch auszugsweise,
***	ist nur mit ausdruecklicher schriftlicher Genehmigung der YARD
***	Software GmbH zulaessig.
***
***	Ebenso ist die Speicherung dieser Datei auf Datentraegern wie z.B.
***	Festplatten, Disketten, Magnetbaendern, CDs und aehnlichen, auch 
***	auszugsweise, nur mit ausdruecklicher schriftlicher Genehmigung der 
***	YARD Software GmbH zulaessig.
***
***	Zuwiderhandlungen unterliegen den Strafbestimmungen des Urheberrechts-
***	gesetzes.
***
******************************************************************************/

import java.io.*;
import java.net.*;
import java.sql.*;

/******************************************************************************
***
***	YARD-JDBC demo program.
***
******************************************************************************/

public class demo
{
	public static void main(String[] argv)
	{
		try
		{
			/*
			**	Adjust the url to yout local settings.
			**	In general its built like "jdbc:yardjdbc://<hostname>[:<port>]/<dbsname>/<dbname>"
			**	Use upper case character for the database name.
			*/

			String url = "jdbc:yardjdbc://yard:2000/yard3/RENTAL";
			Class.forName ("yardjdbc.YardDriver");
			Connection con = DriverManager.
				getConnection(url,"DBA","");

			doselect(con,"select * from customer");
		}
		catch (ClassNotFoundException e)
		{
			System.err.println("Class not found");
			System.exit(1);
		}
		catch (SQLException e)
		{
			dberror("main",e);
			System.exit(1);
		}
	}

	/*
	**	perform a SELECT statement
	*/

	static void doselect(Connection con, String stmt)
	{
		int ret;

		try
		{
			PreparedStatement st = con.prepareStatement(stmt);
			ResultSet c = st.executeQuery();

			dumpResultSet(c);
		}
		catch (SQLException e)
		{
			dberror("doselect("+stmt+")",e);
			System.exit(0);
		}
	}

	/*
	**	perform a non SELECT statement
	*/

	static void dosql(Connection con, String stmt)
	{
		int ret;

		try
		{
			Statement st = con.createStatement();

			ret = st.executeUpdate(stmt);
		}
		catch (SQLException e)
		{
			dberror("dosql("+stmt+")",e);
			System.exit(0);
		}
	}

	static void dumpResultSet(ResultSet rs) throws SQLException
	{
		ResultSetMetaData md = rs.getMetaData();
		int i, cols = md.getColumnCount();
		int row = 0;

		for (i=0;i<cols;i++)
			System.out.println("Column " + i + ": " +
				md.getColumnName(i+1));
		while (rs.next())
		{
			System.out.print(row+++": ");
			for (i=0;i<md.getColumnCount();i++)
				System.out.print(rs.getString(i+1)+" ");
			System.out.println();
		}
	}

	/*
	**	Report the SQLCODE if an error occurs
	*/

	static void dberror(String loc, SQLException e)
	{
		System.err.println(loc + ": ");
		e.printStackTrace();
		do
		{
			System.err.println(e.getErrorCode()+
				"("+e.getSQLState()+
				")"+": "+e.getMessage());
			e = e.getNextException();
		}
		while (e != null);
	}
}
