/******************************************************************************
***
***				COPYRIGHT
***
***	Die Daten in dieser Datei sind Eigentum der 
***
***	YARD Software GmbH in 5000 Koeln 90, Deutschland.
***
***	Diese Datei gilt als Werk im Sinne des Urheberrechtsgesetzes und ist
***	damit urheberrechtlich geschuetzt. Jede Verwertung, insbesondere die
***	Vervielfaeltigung, Aenderung und Weitergabe, auch auszugsweise,
***	ist nur mit ausdruecklicher schriftlicher Genehmigung der YARD
***	Software GmbH zulaessig.
***
***	Ebenso ist die Speicherung dieser Datei auf Datentraegern wie z.B.
***	Festplatten, Disketten, Magnetbaendern, CDs und aehnlichen, auch 
***	auszugsweise, nur mit ausdruecklicher schriftlicher Genehmigung der 
***	YARD Software GmbH zulaessig.
***
***	Zuwiderhandlungen unterliegen den Strafbestimmungen des Urheberrechts-
***	gesetzes.
***
******************************************************************************/




/******************************************************************************
***
***	Defines for blobs and text data types in Client processes
***
******************************************************************************/

#ifndef _SQLBLOB_H_
#define _SQLBLOB_H_

static char *sqlblob_h_sccsid = "@(#) sqlblob.h 4.2 98/11/05 16:44:41";

/*
 *	values for b_type
 */
#define BLOB_MEMORY	1	/* locate blob in memory 		*/
#define BLOB_SYSMEM	2	/* locate blob in memory and let 	*/
				/* runtime system allocate memory	*/
#define BLOB_FILE	3	/* locate blob in file 			*/
#define BLOB_FILENAME	4	/* locate blob in named file 		*/
#define BLOB_USER	5	/* locate blob from user func  		*/
#define BLOB_SIZE	6	/* get only size of blob		*/
#define BLOB_CHARBUF	7	/* for internal use only 		*/


#ifdef CLI

#define MAX_MEMSIZE	32000

/*
 *	blob structure
 */
struct blob
{
	long b_indicator;	/* indicator flag */
	int b_flags;		/* flags, currently used for cleaning */
	int b_type;			/* type of blob, see above */
	long b_ntransf;		/* number of bytes transferred */
	long b_size;		/* size of blob,if b_type is BLOB_SIZE*/
	int memcnt;         /* Number of buffers */
	int mempos;         /* actual position into buffer */
	char **mem_buf;		/* pointer to buffer in memory */
	int *mem_size;		/* size of buffer */
};

/*
**	defines for b_flags
*/
#define BLOB_CLEANUP	0x00000001

#else /* CLI */

/*
 *	blob structure
 */
struct blob
{
	long b_indicator;		/* indicator flag */
	short b_type;			/* type of blob, see above */
	long b_ntransf;			/* number of bytes transferred */
	long b_size;			/* size of blob,if b_type is BLOB_SIZE*/
	union				/* union corresponding to b_type */
	{
		struct			/* b_type = BLOB_MEMORY */
		{
			char *b_buf;	/* pointer to buffer in memory */
			long b_size;	/* size of buffer */
		} b_memory;
		struct			/* b_type = BLOB_FILE or BLOB_FILENAME*/
		{
			char *b_fname;	/* filename, if b_type = BLOB_FILENAME*/
			int b_mode;	/* filemode, if b_type = BLOB_FILE */
			int b_fd;	/* filedesc, if b_type = BLOB_FILE */
		} b_file;
		struct			/* b_type = BLOB_USER */
		{
			int (*b_open)();	/* users open() function */
			int (*b_close)();	/* users close() function */
			int (*b_read)();	/* users read() function */
			int (*b_write)();	/* users write() function */
			char *b_buf;		/* buffer to read from or */
						/* write in */
			long b_ntorw;		/* number of bytes to read */
						/* or to write */
			char *b_env;		/* pointer for users own use */
		} b_user;
	} b_loc;
};

/*
 *	defines for union elements
 */
#define	mem_buf		b_loc.b_memory.b_buf
#define	mem_size	b_loc.b_memory.b_size
#define	file_name	b_loc.b_file.b_fname
#define	file_mode	b_loc.b_file.b_mode
#define	file_fd		b_loc.b_file.b_fd
#define	user_open	b_loc.b_user.b_open
#define	user_close	b_loc.b_user.b_close
#define	user_read	b_loc.b_user.b_read
#define	user_write	b_loc.b_user.b_write
#define	user_buf	b_loc.b_user.b_buf
#define	user_ntorw	b_loc.b_user.b_ntorw
#define	user_env	b_loc.b_user.b_env

typedef struct blob BLOB;

#endif /* CLI */

/*
 *	typedefs for blob structure
 */
typedef struct blob blob_t;

#endif /* _SQLBLOB_H_ */
