/******************************************************************************
***
***				COPYRIGHT
***
***	Die Daten in dieser Datei sind Eigentum der 
***
***	YARD Software GmbH in 5000 Koeln 90, Deutschland.
***
***	Diese Datei gilt als Werk im Sinne des Urheberrechtsgesetzes und ist
***	damit urheberrechtlich geschuetzt. Jede Verwertung, insbesondere die
***	Vervielfaeltigung, Aenderung und Weitergabe, auch auszugsweise,
***	ist nur mit ausdruecklicher schriftlicher Genehmigung der YARD
***	Software GmbH zulaessig.
***
***	Ebenso ist die Speicherung dieser Datei auf Datentraegern wie z.B.
***	Festplatten, Disketten, Magnetbaendern, CDs und aehnlichen, auch 
***	auszugsweise, nur mit ausdruecklicher schriftlicher Genehmigung der 
***	YARD Software GmbH zulaessig.
***
***	Zuwiderhandlungen unterliegen den Strafbestimmungen des Urheberrechts-
***	gesetzes.
***
******************************************************************************/

#ifndef _SQLCA_H_
#define _SQLCA_H_

static char *sqlca_h_sccsid = "@(#) sqlca.h 4.4 98/11/05 16:44:41";

/*
 *	SQL COMMUNICATION AREA (sqlca)
 */
struct sqlca
{
    	long sqlcode;		/* Contains the return code pertaining 	*/
				/* to the most recently executed SQL  	*/
				/* Statement.				*/
				/* Return code values are defined as 	*/
				/* follows:				*/
				/* 0 = Successfull execution		*/
				/* +100 = Successfull execution, but 	*/
				/* 	  there was no data to process. */
				/* This occurs in the following 	*/
				/* situations:				*/
				/* FETCH statement when no row was 	*/
				/* fetched.				*/
				/* SELECT statement when the result	*/
				/* table is empty			*/
				/* INSERT with SELECT, DELETE and UPDATE*/	
				/* where the search condition is not	*/
				/* satisfied.				*/
				/* < 0 = An error occurred		*/

    	char sqlerrm[80]; 	/* Errormessage corresponding to 	*/
				/* (negative) 'sqlcode'			*/

    	char sqlerri[40];	/* additional information for 'sqlerrm' */

	char sqldb[11];		/* current database name		*/

	char *sqlmodule;	/* current module name			*/

	char *sqlclient;	/* current client name, set by user	*/

    	char sqlerrp[8];	/* reserved				*/

    	long sqlerrd[6];	/* Additional information about the	*/
				/* execution of the statement		*/
				/* Element 0: reserved 			*/
				/*    	   1: ISAM error or serial value*/
			 	/*	      after INSERT		*/
				/*	   2: number of rows processed  */
				/*	      after an INSERT, UPDATE or*/
				/*	      DELETE statement		*/
				/*	   3: Statement code as defined	*/
				/*	      in 'sqlda.h' after 	*/
				/*	      PREPARE			*/
				/*	   4: Number of SELECT items	*/
				/*	      after PREPARE if the	*/
				/*	      PREPAREd Statement is a	*/
				/*	      SELECT (sqlerrd[3] ==	*/
				/*	      OP_SELECT or OP_SSELECT	*/
				/*	   5: reserved 			*/

	long sqlline;		/* Line number of SQL statement		*/

	char sqlwarn0; 		/* if blank, all other sqlwarn's are 	*/
				/* also blank. If set to 'W', at least 	*/
				/* one of the other warning characters	*/
				/* has been set to 'W'.			*/

	char sqlwarn1; 		/* if 'W' at least one character string	*/
				/* value was truncated when it was 	*/
				/* stored into a host variable.		*/

	char sqlwarn2; 		/* if 'W', at least one NULL value was	*/
				/* elimininated from the argument set	*/
				/* of an aggregate function.		*/

	char sqlwarn3; 		/* reserved				*/

	char sqlwarn4; 		/* if 'W', the last Statement PREPAREd	*/
				/* has no WHERE clause			*/ 

	char sqlwarn5; 		/* if 'W', the nummer of occurrences    */
				/* from an ALLOCATE DESCRIPTOR      	*/
				/* statement is lower than the number	*/
				/* of items from DESCRIBE.		*/

	char sqlwarn6; 		/* if 'W', the PRECISION or SCALE 	*/
				/* value specified in a SET DESCRIPTOR	*/
				/* statement will be ignored.         	*/

	char sqlwarn7; 		/* if 'W', a statement id has been	*/
				/* overwritten				*/
	char sqlwarn8; 		/* if 'W', a cursor is destroyed by	*/
				/* overwriting the corresponding	*/
				/* statement id				*/
	char sqlwarn9; 		/* reserved				*/
	char sqlwarn10; 	/* reserved				*/
};

/*
 *	Extern declaration of sqlca structure.
 */
extern struct sqlca sqlca;

/*
 * 	Global variable to hold the SQLCODE. Is equal to 'sqlca.sqlcode'
 */
extern long SQLCODE;

/*
 * 	Global variable to hold the ISAM error. Is equal to 'sqlca.sqlerrd[1]'
 */
extern long ISAMCODE;

/*
 * 	Global variable to hold the main warning 'sqlwarn0'.
 *	Is equal to 'sqlca.sqlwarn0'
 */
extern char SQLMAINWARN;

/*
 *	Define for SQLCODE = +100
 */
#define SQLNOTFOUND 100

#endif /* _SQLCA_H_ */
