/******************************************************************************
***
***				COPYRIGHT
***
***	Die Daten in dieser Datei sind Eigentum der 
***
***	YARD Software GmbH in 5000 Koeln 90, Deutschland.
***
***	Diese Datei gilt als Werk im Sinne des Urheberrechtsgesetzes und ist
***	damit urheberrechtlich geschuetzt. Jede Verwertung, insbesondere die
***	Vervielfaeltigung, Aenderung und Weitergabe, auch auszugsweise,
***	ist nur mit ausdruecklicher schriftlicher Genehmigung der YARD
***	Software GmbH zulaessig.
***
***	Ebenso ist die Speicherung dieser Datei auf Datentraegern wie z.B.
***	Festplatten, Disketten, Magnetbaendern, CDs und aehnlichen, auch 
***	auszugsweise, nur mit ausdruecklicher schriftlicher Genehmigung der 
***	YARD Software GmbH zulaessig.
***
***	Zuwiderhandlungen unterliegen den Strafbestimmungen des Urheberrechts-
***	gesetzes.
***
******************************************************************************/

#ifndef _SQLDA_H_
#define _SQLDA_H_

static char *sqlda_h_sccsid = "@(#) sqlda.h 4.5 98/11/05 16:44:41";

#define MAXTABNAME	18 
#define MAXCOLNAME	18 
#define MAXUSRNAME	15 
#define MAXDESCNAME	18 


/*
**	SQL HOST AREA (sqlha)
*/
struct sqlha
{
	short type;		/* variable type		*/
#ifdef CLI
	long length;		/* length in bytes for char 	*/
				/* and varchar 			*/
#else /* CLI */
	short length;		/* length in bytes for char, 	*/
				/* varchar and blob		*/
#endif
	char *data;		/* pointer to data		*/
#ifdef CLI
	long *indicator;	/* pointer to indicator		*/
#else /* CLI */
	short *indicator;	/* pointer to indicator		*/
#endif
	char tabname[MAXTABNAME + 1]; 	/* table name			*/
	char colname[MAXCOLNAME + 1];	/* column name or display label	*/
	char usrname[MAXUSRNAME + 1];	/* user name		        */
#ifdef CLI
	long precision;		/* precison for all types	*/
#else /* CLI */
	short precision;	/* precison of decimal or	*/
				/* money type       		*/
#endif
	short scale;		/* scale of decimal/money type	*/
	short nulls;		/* 1 = nulls allowed, 		*/
				/* 0 = nulls not allowed 	*/
	long rowid;		/* rowid of table 'tabname'	*/
				/* rowid is -1 for expressions	*/
#ifdef CLI
	short hflags;		/* flags, see values below 	*/
	char *blob;		/* internally used for blobs	*/
#endif /* CLI */
};


#ifdef CLI
/*
**	Bitflags for hflags
*/
#define H_BIND		0x01	/* column was bind by SQLBindCol() 	*/
#define H_DATA		0x02	/* column was retrieved by SQLGetData() */
#define H_PARAM		0x04    /* column was set by SQLSetParam() 	*/
#define H_BLOB		0x08    /* the field blob (sqlha) is set 	*/
				/* or allocated 			*/
#define H_MEM		0x10    /* data pointer was allocated */
#endif /* CLI */

/*
**	typedef for sqlha
*/
typedef struct sqlha sqlha_t;


/*
**	SQL DESCRIPTOR AREA (sqlda)
*/
struct sqlda
{
	long hostcnt;		/* number of host area 		*/
				/* structures			*/
	short stmtop;		/* opcode for statements	*/
	short opflag;		/* flag for opcode used for	*/
				/* select statements		*/
	char *descname;		/* name of descriptor		*/
	sqlha_t *sqlha;		/* pointer to beginning of 	*/
				/* 'hostcnt' number of 		*/
				/* host area structures		*/
};


/*
**	typedef for sqlda
*/
typedef struct sqlda sqlda_t;


/*
** 	macros for statements opcodes.
**	values are available after 
**	-> PREPARE in sqlca.sqlerrd[3]
**	-> DESCRIBE in 'sqlda.stmtop'
*/
#define OP_INSERT	1	/*	INSERT statement		*/
#define OP_DELETE	2	/*	DELETE statement		*/
#define OP_UPDATE	3	/*	UPDATE statement		*/

#define OP_SELECT	6	/*	SELECT statement		*/
#define OP_EXECUTE	7	/*	EXECUTE (for internal use)	*/
#define OP_CREATETAB	8	/* 	CREATE TABLE statement		*/
#define OP_CREATEDB	9	/* 	CREATE DATABASE	statement	*/
#define OP_OPENDB	10	/* 	DATABASE statement		*/
#define OP_DROPTAB	11	/* 	DROP TABLE statement		*/
#define OP_OPENCUR	12	/*	OPEN CURSOR (for internal use)	*/
#define OP_FETCH	13	/*	FETCH (for internal use)	*/
#define OP_CREATEIND	14	/* 	CREATE INDEX statement		*/
#define OP_DROPIND	15	/* 	DROP INDEX statement		*/
#define OP_CREATEVIEW	16	/* 	CREATE VIEW statement		*/

#define OP_CREATEDOM	18	/* 	CREATE DOMAIN statement		*/
#define OP_ALTER	19	/* 	ALTER TABLE statement		*/
#define OP_RENAME	20	/* 	RENAME table or column		*/
#define OP_GRANT	21	/* 	GRANT statement			*/
#define OP_REVOKE	22	/* 	REVOKE statement		*/
#define OP_LOAD		23	/* 	LOAD statement			*/
#define OP_UNLOAD	24	/* 	UNLOAD statement		*/

#define OP_DROPVIEW	27	/* 	DROP VIEW statement		*/
#define OP_DROPDB	28	/* 	DROP DATABASE statement		*/

#define OP_COMMIT	30	/* 	COMMIT statement		*/
#define OP_ROLLBACK	31	/* 	ROLLBACK statement		*/

#define OP_CREATESYN	35	/* 	CREATE SYNONYM statement	*/
#define OP_DROPSYN	36	/* 	DROP SYNONYM statement		*/
#define OP_LOCKROW	37	/* 	LOCK ROW statement		*/
#define OP_LOCKTAB	38	/* 	LOCK TABLE statement		*/
#define OP_UNLOCKTAB	39	/* 	UNLOCK TABLE statement 		*/

				
#define OP_CLOSEDB	41	/* 	CLOSE DATABASE statement	*/

#define OP_CRUSER	43	/* 	CREATE USER statement		*/
#define OP_DROPUSER	44	/* 	DROP USER statement		*/
#define OP_DROPDOMAIN	45	/* 	DROP DOMAIN statement 		*/
#define OP_SETLOG	46	/* 	SET LOGFILE statement		*/
#define OP_SSELECT	47	/*	SELECT positioned		*/
#define OP_SETISO	48	/*	SET ISOLATION statement		*/
#define OP_SETWAIT	49	/*	SET LOCK MODE WAIT statement	*/
#define OP_SETOPT	50	/*	SET OPTIMIZER statement		*/

#define OP_UPDSTAT	52	/*	UPDATE STATISTICS		*/
#define OP_ALUSER	53	/*	ALTER USER			*/
#define OP_CRDOC	54	/*	CREATE DOCUMENT			*/
#define OP_ALDOC	55	/*	ALTER DOCUMENT			*/
#define OP_DRDOC	56	/*	DROP DOCUMENT			*/
#define OP_SENDBLOB	57


#define OP_CREATE_FUNC	60	/*	CREATE FUNCTION			*/
#define OP_CREATE_PROC	61	/*	CREATE PROCEDURE		*/
#define OP_DROP_FUNC	62	/*	DROP FUNCTION			*/
#define OP_DROP_PROC	63	/*	DROP PROCEDURE			*/
#define OP_CREATE_TRIG	64	/*	CREATE TRIGGER			*/
#define OP_DROP_TRIG	65	/*	DROP TRIGGER			*/
#define OP_CALL_PROC	66	/*	CALL	 			*/
#define OP_SAVEPOINT	67	/*	SAVEPOINT			*/
#define OP_RELEASE	68	/*	RELEASE SAVEPOINT		*/
#define OP_CALL_FUNC	69	/*	CALL				*/


/*
** 	opcode bit flags, only used for dynamic statements
**	values are available after DESCRIBE in 'opflag'
*/
#define ST_ORDER	0x00000001	/*	SELECT with ORDER BY	*/
#define ST_UNION	0x00000002	/*	SELECT with UNION	*/
#define ST_FORUPD	0x00000004	/*	SELECT with FOR UPDATE	*/
#define ST_UNBIND	0x00000008	/*	at least one dynamic 	*/
					/*	parameter		*/
#define ST_NOWHERE	0x00000010	/*	SQL statement 		*/
					/*	without WHERE clause 	*/
#define ST_INTOTMP	0x00000020	/*	SELECT with INTO TEMP	*/
#define ST_UPDATABLE	0x00000040	/*	SELECT is updatable	*/


/*
**	defines for testing a specific bit flag
*/
#define  SEL_ORDER(opflag)	(opflag & ST_ORDER)
#define  SEL_UNION(opflag)	(opflag & ST_UNION)
#define  SEL_UPDATE(opflag)	(opflag & ST_FORUPD)
#define  SEL_TEMP(opflag)	(opflag & ST_INTOTMP)
#define  DYN_UNBIND(opflag)	(opflag & ST_UNBIND)
#define  STMT_NOWHERE(opflag)	(opflag & ST_NOWHERE)

#endif /* _SQLDA_H_ */
