/******************************************************************************
***
***				COPYRIGHT
***
***	Die Daten in dieser Datei sind Eigentum der 
***
***	YARD Software GmbH in 5000 Koeln 90, Deutschland.
***
***	Diese Datei gilt als Werk im Sinne des Urheberrechtsgesetzes und ist
***	damit urheberrechtlich geschuetzt. Jede Verwertung, insbesondere die
***	Vervielfaeltigung, Aenderung und Weitergabe, auch auszugsweise,
***	ist nur mit ausdruecklicher schriftlicher Genehmigung der YARD
***	Software GmbH zulaessig.
***
***	Ebenso ist die Speicherung dieser Datei auf Datentraegern wie z.B.
***	Festplatten, Disketten, Magnetbaendern, CDs und aehnlichen, auch 
***	auszugsweise, nur mit ausdruecklicher schriftlicher Genehmigung der 
***	YARD Software GmbH zulaessig.
***
***	Zuwiderhandlungen unterliegen den Strafbestimmungen des Urheberrechts-
***	gesetzes.
***
******************************************************************************/

#ifndef _SQLTYPES_H_
#define _SQLTYPES_H_

static char *sqltypes_h_sccsid = "@(#) sqltypes.h 4.6 98/11/05 16:44:40";

/*
**	SQL data types, as defined in the X/Open SQL specification
*/
#define SQL_CHAR		1
#define SQL_NUMERIC		2
#define SQL_DECIMAL		3
#define SQL_INTEGER		4
#define SQL_SMALLINT		5
#define SQL_FLOAT		6
#define SQL_REAL		7
#define SQL_DOUBLE		8	/* = SQL_FLOAT	*/
#define SQL_DATETIME		9	
#define SQL_INTERVAL		10

#define SQL_VARCHAR		12

/*
**	Begin of YARD-SQL specific data types
*/
#define SQL_MONEY		-1
#define SQL_SERIAL		-2
#define SQL_BLOB		-3
#define SQL_TEXT		-4
#ifdef CLI
#define YARD_SQL_DATE		-5
#else
#define SQL_DATE		-5
#endif

#define SQL_NOTYPE		-20

/*
**	datetime and interval qualifiers
*/

#define SQL_DATETIME_DATE	1
#define SQL_DATETIME_TIME	2
#define SQL_DATETIME_TIMESTAMP	3

#define SQL_INTERVAL_DA		3	/*	00011	*/
#define SQL_INTERVAL_DA_TO_HO	8	/*	01000	*/
#define SQL_INTERVAL_DA_TO_MI	9	/*	01001	*/
#define SQL_INTERVAL_DA_TO_SE	10	/*	01010	*/
#define SQL_INTERVAL_HO		4	/*	00100	*/
#define SQL_INTERVAL_HO_TO_MI	11	/*	01011	*/
#define SQL_INTERVAL_HO_TO_SE	12	/*	01100	*/
#define SQL_INTERVAL_MI		5	/*	00101	*/
#define SQL_INTERVAL_MI_TO_SE	13	/*	01101	*/
#define SQL_INTERVAL_MO		2	/*	00010	*/
#define SQL_INTERVAL_SE		6	/*	00110	*/
#define SQL_INTERVAL_YE		1	/*	00001	*/
#define SQL_INTERVAL_YE_TO_MO	7	/*	00111	*/

/*
**	end of YARD-SQL data type codes
*/

/*
**	C data types for use in an sqlda structure and
**	in GET/SET DESCRIPTOR statements (dynamic SQL)
*/
#define C_CHAR		256	/* corresponds to SQL_CHAR		   */
#define C_SHORT	 	257	/* corresponds to SQL_SMALLINT		   */
#define C_INT		258	/* corresponds to SQL_INTEGER		   */
#define C_LONG		259	/* corresponds to SQL_INTEGER		   */
#define C_FLOAT		260	/* corresponds to SQL_REAL		   */
#define C_DOUBLE	261	/* corresponds to SQL_DOUBLE and SQL_FLOAT */
#define C_DECIMAL	262	/* corresponds to SQL_NUMERIC, SQL_DECIMAL */
				/* and SQL_MONEY 			   */
#define C_VARCHAR	263	/* corresponds to SQL_VARCHAR		   */
#define C_BLOB        	264	/* corresponds to SQL_BLOB and SQL_TEXT	   */
#define C_INTERVAL     	265	/* for future use 			   */
#define C_DATETIME     	266	/* for future use 			   */
#define C_STRUCT        267	/* for internal use only 		   */
#define C_STRING        268	/* handles SQL_CHAR and SQL_VARCHAR with   */
				/* with no trailing blanks		   */
#define C_FIXCHAR       269	/* handles SQL_CHAR and SQL_VARCHAR with   */
				/* with no terminating null byte 	   */

/*
**	Special Function Types for internal use
*/
#define SQF_NULL		100
#define SQF_TODAY		101
#define SQF_USER		102
#define SQF_SYSTEM_USER		103

/*
**	defines for data type management
*/
#define IS_CHAR_TYPE(x) ((x) == C_CHAR || (x) == C_VARCHAR || (x) == C_STRING \
				|| (x) == C_FIXCHAR || (x) == SQL_CHAR \
				|| (x) == SQL_VARCHAR)

#define IS_INT_TYPE(x) ((x) == C_SHORT || (x) == C_INT || (x) == C_LONG \
				|| (x) == SQL_INTEGER || (x) == SQL_SMALLINT)

#define IS_BLOB_TYPE(x) ((x) == C_BLOB || (x) == SQL_BLOB || (x) == SQL_TEXT)

/******************************************************************************
***
***
***
******************************************************************************/

#define ISYEARMONTHIVAL(t)	((t)==SQL_INTERVAL_YE||(t)==SQL_INTERVAL_YE_TO_MO)
#define ISVALIDQUALI(q)		(((q)>=SQL_INTERVAL_YE&&(q)<=SQL_INTERVAL_MI_TO_SE)?1:0)

/*
**	macros for NULL values in host variables
*/
#define NULL_CHAR 	'\0'		/* SQL_CHAR, SQL_VARCHAR	  */
#define NULL_SHORT 	(-32768)	/* SQL_SMALLINT			  */
#define NULL_LONG 	(0x80000000)	/* SQL_INTEGER, SQL_SERIAL	  */
#define NULL_FLOAT 	"\x77\x77\x77\x77"
					/* SQL_REAL (and SMALLFLOAT)	  */
#define NULL_DOUBLE 	"\x77\x77\x77\x77\x77\x77\x77\x77"
					/* SQL_FLOAT, SQL_DOUBLE	  */

/*
**	macros for checking NULL values in host variables
*/
#define IS_NULL_CHAR(hv)	(((hv)[0] == NULL_CHAR) ? 1 : 0)
#define IS_NULL_STRING(hv)	(((hv)[0] == ' ') ? 1 : 0)
#define IS_NULL_SHORT(hv)	(((hv) == NULL_SHORT) ? 1 : 0)
#define IS_NULL_LONG(hv)	(((hv) == NULL_LONG) ? 1 : 0)
#define IS_NULL_FLOAT(hv)	((!memcmp(&(hv),NULL_FLOAT,4)) ? 1 : 0)
#define IS_NULL_DOUBLE(hv)	((!memcmp(&(hv),NULL_DOUBLE,8)) ? 1 : 0)
#define IS_NULL_DECIMAL(hv)	(((hv)->dec_pos == -1) ? 1 : 0)
#define IS_NULL_BLOB(hv)	(((hv)->b_indicator == -1) ? 1 : 0)

#define CODEDATATYPE(t,q)	(((t)==SQL_DATETIME)?((q)+20):(((t)==SQL_INTERVAL)?((q)+30):(t)))
#define DECODEDATATYPE(t)	(((t)>=20)?(((t)>=30)?SQL_INTERVAL:SQL_DATETIME):(t))
#define DECODEDATAQUALI(t)	(((t)>=20)?(((t)>=30)?((t)-30):((t)-20)):(0))

#endif	/* _SQLTYPES_H_ */
