/**
 * You may distribute under the terms of either the GNU General Public
 * License or the Artistic License, as specified in the Perl README file.
 */

#include "clibasic.h"
#include "sql.h"
#include "sqlext.h"
#include "dbdimp.h"


DBISTATE_DECLARE;

SV *dbd_errnum = NULL;
SV *dbd_errstr = NULL;
SV *dbd_state = NULL;

MODULE = DBD::YARD	PACKAGE = DBD::YARD

REQUIRE:	1.9
PROTOTYPES:	DISABLE

BOOT:
    items = 0;	/* avoid 'unused variable' warning */
    DBISTATE_INIT;
    /* XXX tis interface will change: */
    DBI_IMP_SIZE("DBD::YARD::dr::imp_data_size", sizeof(imp_drh_t));
    DBI_IMP_SIZE("DBD::YARD::db::imp_data_size", sizeof(imp_dbh_t));
    DBI_IMP_SIZE("DBD::YARD::st::imp_data_size", sizeof(imp_sth_t));
    dbd_dr_init(DBIS);

void
errstr(h)
    SV *	h
    CODE:
    /* called from DBI::var TIESCALAR code for $DBI::errstr     */
    D_imp_xxh(h);
    ST(0) = sv_mortalcopy(DBIc_ERRSTR(imp_xxh));


MODULE = DBD::YARD	PACKAGE = DBD::YARD::dr

void
disconnect_all(drh)
	SV *        drh
	CODE:
	if (!dirty && !SvTRUE(perl_get_sv("DBI::PERL_ENDING",0))) {
		/* disconnect is done by later socket close	*/
		XSRETURN(0);
	}
	/* perl_destruct with perl_destruct_level and $SIG{__WARN__} set    */
	/* to a code ref core dumps when sv_2cv triggers warn loop.         */
	if (perl_destruct_level)
	perl_destruct_level = 0;
	XST_mIV(0, 1);

void
init(drh)
    SV *	drh
    CODE:
    ST(0) = dbd_dr_driver(drh) ? &sv_yes : &sv_no;


MODULE = DBD::YARD    PACKAGE = DBD::YARD::db

void
connect(dbh, dbname, uid, pwd)
	SV   *dbh
	char *dbname
	char *uid
	char *pwd
	CODE:
	ST(0) = dbd_db_connect(dbh, dbname, uid, pwd) ? &sv_yes : &sv_no;

void
commit(dbh)
	SV *        dbh
	CODE:
	ST(0) = dbd_db_commit(dbh) ? &sv_yes : &sv_no;

void
rollback(dbh)
	SV *        dbh
	CODE:
	ST(0) = dbd_db_rollback(dbh) ? &sv_yes : &sv_no;

#******************************************************************************
#**
#**	Store/Fetch a connection attribute
#**
#*****************************************************************************/

void
STORE(dbh, keysv, valuesv)
	SV *        dbh
	SV *        keysv
	SV *        valuesv
	CODE:
	if (!dbd_db_STORE(dbh, keysv, valuesv))
		if (!DBIS->set_attr(dbh, keysv, valuesv))
			ST(0) = &sv_no;

void
FETCH(dbh, keysv)
	SV *        dbh
	SV *        keysv
	CODE:
	SV *valuesv = dbd_db_FETCH(dbh, keysv);
	if (!valuesv)
		valuesv = DBIS->get_attr(dbh, keysv);
	ST(0) = valuesv;  


void
disconnect(dbh)
	SV *        dbh
	CODE:
	D_imp_dbh(dbh);
	if ( !DBIc_ACTIVE(imp_dbh) ) {
		XSRETURN_YES;
	}
	/* Check for disconnect() being called whilst refs to cursors       */
	/* still exists. This needs some more thought.                      */
	if (DBIc_ACTIVE_KIDS(imp_dbh) && DBIc_WARN(imp_dbh) && !dirty) {
		warn("disconnect(%s) invalidates %d active cursor(s)",
		SvPV(dbh,na), (int)DBIc_ACTIVE_KIDS(imp_dbh));
	}
	ST(0) = dbd_db_disconnect(dbh) ? &sv_yes : &sv_no;


void
DESTROY(dbh)
	SV *        dbh
	CODE:
	D_imp_dbh(dbh);
	ST(0) = &sv_yes;
	if (!DBIc_IMPSET(imp_dbh)) {
		if (DBIc_WARN(imp_dbh) && !dirty && dbis->debug >= 2)
		warn("Database handle %s DESTROY ignored - never set up",
		SvPV(dbh,na));
	}
	else {
		if (DBIc_ACTIVE(imp_dbh)) {
			if (DBIc_WARN(imp_dbh) && !dirty)
				warn("Database handle destroyed without explicit disconnect");
			dbd_db_disconnect(dbh);
		}
		dbd_db_destroy(dbh);
	}


MODULE = DBD::YARD    PACKAGE = DBD::YARD::st

void
prepare(sth, statement, attribs=Nullsv)
	SV *        sth
	char *      statement
	SV *	attribs
	CODE:
	DBD_ATTRIBS_CHECK("_prepare", sth, attribs);
	ST(0) = dbd_st_prepare(sth, statement, attribs) ? &sv_yes : &sv_no;


void
rows(sth)
	SV *        sth
	CODE:
	XST_mIV(0, dbd_st_rows(sth));


void
bind_param( sth, param, value, attribs=Nullsv)
	SV *	sth
	SV *	param
	SV *	value
	SV *	attribs
	CODE:
	{
	D_imp_sth(sth);

	if (!dbd_st_bindsv(imp_sth, SvIV(param), value))
		XSRETURN_UNDEF;
	}


void
bind_param_inout(sth, param, value_ref, maxlen, attribs=Nullsv)
    SV *        sth
    SV *        param
    SV *        value_ref
    IV          maxlen
    SV *        attribs
    CODE:
    ST(0) = &sv_undef;

#******************************************************************************
#**
#**	Execute a previously prepared statement.
#**
#*****************************************************************************/

void
execute(sth, ...)
	SV *        sth
	CODE:
	D_imp_sth(sth);
	int retval;

	if (items > 1)
	{
		int i;

		for(i = 1; i < items; i++)
			if (!dbd_st_bindsv(imp_sth, i, ST(i)))
				XSRETURN_UNDEF;
	}

	/* describe and allocate storage for results */

	retval = dbd_st_execute(sth, imp_sth);
	if ( retval < -1 ) {
		XST_mUNDEF( 0 );
		} else { 
		if ( retval == 0 ) {
			XST_mPV( 0, "0E0" );
		} else {
			XST_mIV( 0, retval );
		}
	}


#******************************************************************************
#**
#**	Return a row as a reference to an array
#**
#*****************************************************************************/

void
fetch(sth)
        SV *    sth
        CODE:
        D_imp_sth(sth);
        AV *av = dbd_st_fetch(imp_sth);
        ST(0) = (av) ? sv_2mortal(newRV((SV *)av)) : &sv_undef;

#******************************************************************************
#**
#**	Return a row as an array
#**
#*****************************************************************************/

void
fetchrow(sth)
	SV *	sth
	PPCODE:
	D_imp_sth(sth);
	AV *av = dbd_st_fetch(imp_sth);
	if (av)
	{
		int num_fields = AvFILL(av)+1;
		int i;

		EXTEND(sp, num_fields);
		for(i = 0; i < num_fields; ++i)
		{
			PUSHs(AvARRAY(av)[i]);
		}
	}


void
blob_read(sth, field, offset, len, destrv=Nullsv, destoffset=0)
    SV *        sth
    int field
    long        offset
    long        len
    SV *        destrv
    long	destoffset
    CODE:
    ST(0) = &sv_undef;


#******************************************************************************
#**
#**	Store a statement attribute
#**
#*****************************************************************************/

void
STORE(sth, keysv, valuesv)
    SV *        sth
    SV *        keysv
    SV *        valuesv
    CODE:
    ST(0) = &sv_yes;
    if (!dbd_st_STORE(sth, keysv, valuesv))
        if (!DBIS->set_attr(sth, keysv, valuesv))
            ST(0) = &sv_no;

#******************************************************************************
#**
#**	Fetch a statement attribute
#**
#*****************************************************************************/

void
FETCH(sth, keysv)
    SV *        sth
    SV *        keysv
    CODE:
    SV *valuesv = dbd_st_FETCH(sth, keysv);
    if (!valuesv)
        valuesv = DBIS->get_attr(sth, keysv);
    ST(0) = valuesv;    /* dbd_st_FETCH did sv_2mortal  */

void
finish(sth)
	SV *        sth
	CODE:
	D_imp_sth(sth);
	D_imp_dbh_from_sth;
	if (!DBIc_ACTIVE(imp_dbh)) {
		/*Either an explicit disconnect() or global destruction       */
		/*has disconnected us from the database. Finish is meaningless*/
		/*XXX warn */
		XSRETURN_YES;
	}
	if (!DBIc_ACTIVE(imp_sth)) {
		/* No active statement to finish        */
		XSRETURN_YES;
	}
	ST(0) = dbd_st_finish(imp_sth) ? &sv_yes : &sv_no;


void
DESTROY(sth)
	SV *        sth
	CODE:
	D_imp_sth(sth);
	ST(0) = &sv_yes;
	if (!DBIc_IMPSET(imp_sth)) {        /* was never fully set up       */
		if (DBIc_WARN(imp_sth) && !dirty)
			warn("Statement handle %s DESTROY ignored - never set up",
		SvPV(sth,na));
		return;
	}
	if (DBIc_ACTIVE(imp_sth)) {
		dbd_st_finish(imp_sth);
	}
	dbd_st_destroy(imp_sth);

# end of YARD.xs
